#ifdef FF_HAVE_WAYLAND

/* Generated by wayland-scanner 1.24.0 */

/*
 * SPDX-FileCopyrightText: 2008-2011 Kristian Høgsberg
 * SPDX-FileCopyrightText: 2010-2011 Intel Corporation
 * SPDX-FileCopyrightText: 2012-2013 Collabora, Ltd.
 * SPDX-FileCopyrightText: 2015 Sebastian Kügler <sebas@kde.org>
 * SPDX-FileCopyrightText: 2021 Méven Car <meven.car@enioka.com>
 *
 * SPDX-License-Identifier: MIT-CMU
 */

    #include <stdbool.h>
    #include <stdlib.h>
    #include <stdint.h>
    #include <wayland-util.h>

extern const struct wl_interface kde_output_device_mode_v2_interface;
extern const struct wl_interface kde_output_device_v2_interface;

static const struct wl_interface* kde_output_device_v2_types[] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    &kde_output_device_v2_interface,
    &kde_output_device_mode_v2_interface,
    &kde_output_device_mode_v2_interface,
};

static const struct wl_message kde_output_device_registry_v2_requests[] = {
    { "stop", "21", kde_output_device_v2_types + 0 },
};

static const struct wl_message kde_output_device_registry_v2_events[] = {
    { "finished", "21", kde_output_device_v2_types + 0 },
    { "output", "21n", kde_output_device_v2_types + 8 },
};

WL_EXPORT const struct wl_interface kde_output_device_registry_v2_interface = {
    "kde_output_device_registry_v2",
    23,
    1,
    kde_output_device_registry_v2_requests,
    2,
    kde_output_device_registry_v2_events,
};

static const struct wl_message kde_output_device_v2_requests[] = {
    { "release", "21", kde_output_device_v2_types + 0 },
};

static const struct wl_message kde_output_device_v2_events[] = {
    { "geometry", "iiiiissi", kde_output_device_v2_types + 0 },
    { "current_mode", "o", kde_output_device_v2_types + 9 },
    { "mode", "n", kde_output_device_v2_types + 10 },
    { "done", "", kde_output_device_v2_types + 0 },
    { "scale", "f", kde_output_device_v2_types + 0 },
    { "edid", "s", kde_output_device_v2_types + 0 },
    { "enabled", "i", kde_output_device_v2_types + 0 },
    { "uuid", "s", kde_output_device_v2_types + 0 },
    { "serial_number", "s", kde_output_device_v2_types + 0 },
    { "eisa_id", "s", kde_output_device_v2_types + 0 },
    { "capabilities", "u", kde_output_device_v2_types + 0 },
    { "overscan", "u", kde_output_device_v2_types + 0 },
    { "vrr_policy", "u", kde_output_device_v2_types + 0 },
    { "rgb_range", "u", kde_output_device_v2_types + 0 },
    { "name", "2s", kde_output_device_v2_types + 0 },
    { "high_dynamic_range", "3u", kde_output_device_v2_types + 0 },
    { "sdr_brightness", "3u", kde_output_device_v2_types + 0 },
    { "wide_color_gamut", "3u", kde_output_device_v2_types + 0 },
    { "auto_rotate_policy", "4u", kde_output_device_v2_types + 0 },
    { "icc_profile_path", "5s", kde_output_device_v2_types + 0 },
    { "brightness_metadata", "6uuu", kde_output_device_v2_types + 0 },
    { "brightness_overrides", "6iii", kde_output_device_v2_types + 0 },
    { "sdr_gamut_wideness", "6u", kde_output_device_v2_types + 0 },
    { "color_profile_source", "7u", kde_output_device_v2_types + 0 },
    { "brightness", "8u", kde_output_device_v2_types + 0 },
    { "color_power_tradeoff", "10u", kde_output_device_v2_types + 0 },
    { "dimming", "11u", kde_output_device_v2_types + 0 },
    { "replication_source", "13s", kde_output_device_v2_types + 0 },
    { "ddc_ci_allowed", "14u", kde_output_device_v2_types + 0 },
    { "max_bits_per_color", "15u", kde_output_device_v2_types + 0 },
    { "max_bits_per_color_range", "15uu", kde_output_device_v2_types + 0 },
    { "automatic_max_bits_per_color_limit", "15u", kde_output_device_v2_types + 0 },
    { "edr_policy", "16u", kde_output_device_v2_types + 0 },
    { "sharpness", "17u", kde_output_device_v2_types + 0 },
    { "priority", "18u", kde_output_device_v2_types + 0 },
    { "auto_brightness", "20u", kde_output_device_v2_types + 0 },
    { "removed", "21", kde_output_device_v2_types + 0 },
    { "hdr_icc_profile_path", "22s", kde_output_device_v2_types + 0 },
    { "hdr_color_profile_source", "22u", kde_output_device_v2_types + 0 },
    { "abm_level", "23u", kde_output_device_v2_types + 0 },
};

WL_EXPORT const struct wl_interface kde_output_device_v2_interface = {
    "kde_output_device_v2",
    23,
    1,
    kde_output_device_v2_requests,
    40,
    kde_output_device_v2_events,
};

static const struct wl_message kde_output_device_mode_v2_events[] = {
    { "size", "ii", kde_output_device_v2_types + 0 },
    { "refresh", "i", kde_output_device_v2_types + 0 },
    { "preferred", "", kde_output_device_v2_types + 0 },
    { "removed", "", kde_output_device_v2_types + 0 },
    { "flags", "19u", kde_output_device_v2_types + 0 },
};

WL_EXPORT const struct wl_interface kde_output_device_mode_v2_interface = {
    "kde_output_device_mode_v2",
    22,
    0,
    NULL,
    5,
    kde_output_device_mode_v2_events,
};

#endif
